part of masamune.list;

/// List item.
class ListItem extends StatelessWidget {
  /// List item.
  ///
  /// [leading]: Leading widget for items.
  /// [title]: The title of the item.
  /// [text]: The text of the item.
  /// [indent]: Title indent.
  /// [dense]: List tile dense.
  /// [padding]: List tile padding.
  /// [trailing]: List trailing.
  /// [onTap]: Processing when tapped.
  const ListItem({
    this.leading,
    this.onTap,
    required this.title,
    this.text,
    this.dense = false,
    this.padding,
    this.indent = 0,
    this.trailing,
    this.textWidth = 200,
    this.selected = false,
    this.tileColor,
    this.selectedColor,
    this.selectedTileColor,
    this.disabledTapOnSelected = false,
  });

  /// Leading widget for items.
  final Widget? leading;

  /// The title of the item.
  final Widget title;

  /// The text of the item.
  final Widget? text;

  /// Title indent.
  final int indent;

  /// List trailing.
  final Widget? trailing;

  /// Horizontal size of the text side.
  final double textWidth;

  /// List tile dense.
  final bool dense;

  /// List tile padding.
  final EdgeInsetsGeometry? padding;

  /// Processing when tapped.
  final VoidCallback? onTap;

  /// If selected, `true`.
  final bool selected;

  /// Background view.
  final Color? tileColor;

  /// Foreground view when in focus.
  final Color? selectedColor;

  /// Background view when in focus.
  final Color? selectedTileColor;

  /// If it is selected, the tap should not respond.
  final bool disabledTapOnSelected;

  @override
  Widget build(BuildContext context) {
    return ListTile(
      leading: _leading(context),
      dense: dense,
      onTap: disabledTapOnSelected && selected ? null : onTap,
      selected: selected,
      tileColor: tileColor,
      selectedTileColor: selectedTileColor,
      contentPadding: padding,
      title: text != null
          ? Row(
              children: [
                Expanded(
                  flex: 1,
                  child: Padding(
                    padding: EdgeInsets.only(
                      left: 20 * indent.toDouble(),
                    ),
                    child: _title(context),
                  ),
                ),
                Container(
                  width: textWidth,
                  child: _text(context),
                )
              ],
            )
          : _title(context),
      trailing: _trailing(context),
    );
  }

  Widget _title(BuildContext context) {
    var child = title;
    if (selected && selectedColor != null) {
      child = IconTheme(
        data: IconThemeData(color: selectedColor),
        child: DefaultTextStyle(
          style: TextStyle(color: selectedColor),
          child: child,
        ),
      );
    }
    if (indent > 0) {
      child = Padding(
        padding: EdgeInsets.only(
          left: 20 * indent.toDouble(),
        ),
        child: child,
      );
    }
    return child;
  }

  Widget? _text(BuildContext context) {
    var child = text;
    if (child == null) {
      return child;
    }
    if (selected && selectedColor != null) {
      child = IconTheme(
        data: IconThemeData(color: selectedColor),
        child: DefaultTextStyle(
          style: TextStyle(color: selectedColor),
          child: child,
        ),
      );
    }
    return child;
  }

  Widget? _leading(BuildContext context) {
    var child = leading;
    if (child == null) {
      return child;
    }
    if (selected && selectedColor != null) {
      child = IconTheme(
        data: IconThemeData(color: selectedColor),
        child: DefaultTextStyle(
          style: TextStyle(color: selectedColor),
          child: child,
        ),
      );
    }
    return child;
  }

  Widget? _trailing(BuildContext context) {
    var child = trailing;
    if (child == null) {
      return child;
    }
    if (selected && selectedColor != null) {
      child = IconTheme(
        data: IconThemeData(color: selectedColor),
        child: DefaultTextStyle(
          style: TextStyle(color: selectedColor),
          child: child,
        ),
      );
    }
    return child;
  }
}
